//  File:  SBW.H       CallBack functions  Date: 24-SEP-2001

#if !defined( _SBW_H )
#define _SBW_H


#include "../usbc/usbcdef.h"


class ErrorInfo
{
public:

    enum ErrorNumber
    { 
		ERROR_NUMBER_ABSENT = -1
	};

    enum AxisNumber
    { 
		AXIS_NUMBER_ABSENT = -1
	};

   ErrorInfo()
   {
      lNumber        = ERROR_NUMBER_ABSENT;
	  cAxis          = AXIS_NUMBER_ABSENT;
	  ulReserved1    = ulReserved2 = 0;
      cOptional[ 0 ] = '\0';  
   }// end of ErrorInfo()

   long lNumber;
   char  cAxis;
   ULONG ulReserved1;
   ULONG ulReserved2;
   TCHAR  cOptional[ 128 ];                 // New
};


//#ifdef __cplusplus
//extern "C" {
//#endif
//typedef unsigned char  InOut;
//typedef InOut   InOutData[IOS/8];

void InitEnd( ConfigData *pConfigData );

void ErrorMessage( ErrorInfo *pTheErrorInfo );

void ViewEnco( RobotData TheEnco );
void ViewJoint( RobotData TheJoint );
void ViewXYZ( RobotData TheXYZ );


#define HOME_NOTIF_START  0xff
#define HOME_NOTIF_FINISH 0x40
//  ucNotifCode
//  HOME_NOTIF_START
//  1...8 Start Homing Axis 
//  HOME_NOTIF_FINISH
void HomingNotif( UCHAR * ucNotifCode );

void ViewErr( RobotData TheErr );
void ViewHomeSwitches( RobotData HomeSwitches );
void ViewTorque( long *lTorque );


// ucEndCode:
// 'A' - group A
// 'B' - group B
// 'G' - gripper
//  0...7 Axis
void MotionEnd( UCHAR *ucEndCode );

class  CMotionStart
{
public:
    UCHAR ucGroup;
	double dSpeed;    // Micrometr/sec
    double dTime;     // Seconds
};// end of class  CMotionStart

void MotionStart( CMotionStart *pCMotionStart );

class DigitalIO
{
private:
     
	ULONG m_ulStatusBitmap;
	ULONG m_ulChangeBitmap;

public:

	DigitalIO( ) { m_ulStatusBitmap = m_ulChangeBitmap = 0; };

	void ClearStatusBitmap( ){ m_ulStatusBitmap = 0; };
    void ClearChangeBitmap( ){ m_ulChangeBitmap = 0; }; 

   void SetStatusBitmap( ULONG ulStatusBitmap )
   {
	 ULONG ulStatBitmap = ulStatusBitmap;
     if( ulStatBitmap == m_ulStatusBitmap )
	 {
		 return;
	 }// end of m_ulStatusBitmap

	 for( int i = 0; i < 32; i++ )
	 {
         ULONG ulBitMask =  1 << i; 
         ULONG ulCurBit = ulStatBitmap & ulBitMask;  
		 BOOL bIsOn =	(ulCurBit) ? TRUE : FALSE;
		 if( ulCurBit == ( m_ulStatusBitmap & ulBitMask ))
             SetDigitalIOStatus ( bIsOn, FALSE, i );
		 else
             SetDigitalIOStatus ( bIsOn, TRUE, i );
	 }// end of for( int i = 0; i < 32; i++ )

	 m_ulStatusBitmap = ulStatusBitmap;
   }// end of void SetStatusBitmap( ULONG ulStatusBitmap )

    void SetChangeBitmap( ULONG ulChangeBitmap ){ m_ulChangeBitmap = ulChangeBitmap; }; 

	ULONG GetStatus( ){ return m_ulStatusBitmap; };
    ULONG GetChanges( ){ return m_ulChangeBitmap; }; 

	// iNumber 0...31
	BOOL SetDigitalIOStatus ( BOOL bStatus, BOOL bIsChanged, int iNumber )
	{
		if( iNumber < 0 || iNumber > 31 )
			return FALSE;

		ULONG ulWorkMask, ulBitMask =  1 << iNumber;

      if( bStatus )
	  {
          ulWorkMask = ulBitMask;
		  m_ulStatusBitmap |= ulWorkMask;
	  }
	  else
	  {
		  ulWorkMask = ~ulBitMask;
		  m_ulStatusBitmap &= ulWorkMask;
	  }
		
      if( bIsChanged )
	  {
          ulWorkMask = ulBitMask;
		  m_ulChangeBitmap |= ulWorkMask;
	  }
	  else
	  {
		  ulWorkMask = ~ulBitMask;
		  m_ulChangeBitmap &= ulWorkMask;
	  }

         return TRUE;
	}// end of BOOL SetDigitalIOStatus ( BOOL bStatus, BOOL bIsChanged, int iNumber )


	// iNumber 0...31
	BOOL GetDigitalIOStatus ( BOOL &bStatus, BOOL &bIsChanged, int iNumber )
	{
		BOOL bStat, bChange;
		if( iNumber < 0 || iNumber > 31 )
			return FALSE;

        ULONG ulBitMask =  1 << iNumber;

		bChange = ( m_ulChangeBitmap & ulBitMask ) ? TRUE : FALSE;
		bStat = ( m_ulStatusBitmap & ulBitMask ) ? TRUE : FALSE;
         
		bIsChanged = bChange;
		bStatus = bStat;
	
		return TRUE;
	}// end of BOOL GetDigitalIOStatus ( BOOL bIsChanged, int iNumber )

};// end of class DigitalIO

void WatchInputs( DigitalIO *pDigitalIO );
void WatchOutputs( DigitalIO *pDigitalIO );

void WatchJoints( RobotData Joints );

void WatchAxisControl( UINT * uiBitmap );

class AnalogIO
{
  private:
      
	UCHAR m_ucValues[ TOTAL_ANALOG_INP ];
	UCHAR m_ucChange[ TOTAL_ANALOG_INP ];


  public: 
	  AnalogIO(){ ClearAllChange( ); ClearAllValues( ); };

	void ClearAllValues( )
	{
        for( int i = 0; i < TOTAL_ANALOG_INP; i++ )
		{
            m_ucValues[ i ] = 0;
		}// end of for( int i = 0; i < TOTAL_ANALOG_INP; i++ )
	}// end of void ClearAllValues( );

    void ClearAllChange( )
	{
        for( int i = 0; i < TOTAL_ANALOG_INP; i++ )
		{
            m_ucChange[ i ] = FALSE;
		}// end of for( int i = 0; i < TOTAL_ANALOG_INP; i++ )
	}// end of void ClearAllChange( );

	void ForceChanges()
	{
        for( int i = 0; i < TOTAL_ANALOG_INP; i++ )
		{
			m_ucChange[ i ] = 1;
		}// end of for( int i = 0; i < TOTAL_ANALOG_INP; i++ )
	};// end of BOOL ForceChanges()

	BOOL WasChanges()
	{
		BOOL bRet = FALSE;
        for( int i = 0; i < TOTAL_ANALOG_INP; i++ )
		{
			if( m_ucChange[ i ] )
			{
                 bRet = TRUE;
				 break;
			}// end of if( m_ucChange[ i ] )
            
		}// end of for( int i = 0; i < TOTAL_ANALOG_INP; i++ )

       return bRet; 
	}// end of BOOL WasChanges()

    BOOL SetAnalogIO( UCHAR ucValue, BOOL bIsChanged, int iAIONumber )
	{
       if( iAIONumber < 0 || iAIONumber > TOTAL_ANALOG_INP - 1 )
		   return FALSE;

       m_ucValues[ iAIONumber ] = ucValue;
       m_ucChange[ iAIONumber ] = bIsChanged;

       return TRUE;
	}// end of BOOL SetAnalogIO( UCHAR ucValue, BOOL bIsChanged, int iAIONumber )

	BOOL GetAnalogIO( UCHAR &ucValue, BOOL &bIsChanged, int iAIONumber )
	{
       if( iAIONumber < 0 || iAIONumber > TOTAL_ANALOG_INP - 1 )
		   return FALSE;

       ucValue = m_ucValues[ iAIONumber ];
       bIsChanged = m_ucChange[ iAIONumber ];

	   return TRUE;
	}// end of BOOL GetAnalogIO( UCHAR &ucValue, BOOL &bIsChanged, int iAIONumber )

	UCHAR GetAnalogIO( int iAIONumber )
	{
       if( iAIONumber < 0 || iAIONumber > TOTAL_ANALOG_INP - 1 )
		   return 0;

	   return m_ucValues[ iAIONumber ];
	}// end of BOOL GetAnalogIO( UCHAR &ucValue, BOOL &bIsChanged, int iAIONumber )

};//end of class AnalogIO


void WatchAnalInputs( AnalogIO  *pAnalogIO );
void WatchAnalOutputs( AnalogIO *pAnalogIO );

void ConfigAvailNotif( ConfigData *pTheConfigData );

//#ifdef __cplusplus
 //};
//#endif

#endif